"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var _1 = require(".");
var flags_1 = require("./flags");
var config_1 = require("./config");
var testConfig = __assign(__assign({}, config_1.defaultConfig), { fileNames: [config_1.DEFAULT_FILE_NAME], includeHeader: false });
describe("Generate io-ts validators", function () {
    test("generates validators for primitive types", function () {
        expect(_1.getValidatorsFromString("type num = number;", testConfig)).toBe("const num = t.number");
        expect(_1.getValidatorsFromString("type str = string;", testConfig)).toBe("const str = t.string");
        expect(_1.getValidatorsFromString("type nil = null;", testConfig)).toBe("const nil = t.null");
    });
    test("generates validators for basic interfaces and object types", function () {
        var inputInterface = "\n    interface Test { foo: number, bar: string }\n  ";
        var inputObjectType = "\n    type Test = { foo: number, bar: string }\n  ";
        var result = "const Test = t.type({foo: t.number, bar: t.string})";
        expect(_1.getValidatorsFromString(inputInterface, testConfig)).toBe(result);
        expect(_1.getValidatorsFromString(inputObjectType, testConfig)).toBe(result);
    });
    test("generates validators for arrays", function () {
        expect(_1.getValidatorsFromString("type arr = string[]", testConfig)).toBe("const arr = t.array(t.string)");
        expect(_1.getValidatorsFromString("type arr = Array<{foo: string}>", testConfig)).toBe("const arr = t.array(t.type({foo: t.string}))");
    });
    test("generates validators for record types", function () {
        expect(_1.getValidatorsFromString("type rec = Record<number, string>", testConfig)).toBe("const rec = t.record(t.number, t.string)");
        expect(_1.getValidatorsFromString("type rec = Record<string, null>", testConfig)).toBe("const rec = t.record(t.string, t.null)");
    });
    test("generates validators for union types", function () {
        expect(_1.getValidatorsFromString("type un = string | number", testConfig)).toBe("const un = t.union([t.string, t.number])");
        expect(_1.getValidatorsFromString("type un = string | number | { foo: string }", testConfig)).toBe("const un = t.union([t.string, t.number, t.type({foo: t.string})])");
    });
    test("optimizes validator for string literal union types", function () {
        expect(_1.getValidatorsFromString("type un = 'foo' | 'bar'", testConfig)).toBe('const un = t.keyof({"foo": null, "bar": null})');
    });
    test("generates validators for intersection types", function () {
        expect(_1.getValidatorsFromString("type inter = { foo: string } | { bar: number } | { foo: number }", testConfig)).toBe("const inter = t.union([t.type({foo: t.string}), t.type({bar: t.number}), t.type({foo: t.number})])");
    });
    test("generates validators for function types", function () {
        expect(_1.getValidatorsFromString("type fn = () => void", testConfig)).toBe("const fn = t.Function");
        expect(_1.getValidatorsFromString("type fn = (s: string, n: number) => (b: boolean) => object", testConfig)).toBe("const fn = t.Function");
    });
    test("generates validators for literal types", function () {
        expect(_1.getValidatorsFromString('type foo = "foo"', testConfig)).toBe('const foo = t.literal("foo")');
        expect(_1.getValidatorsFromString("type one = 1", testConfig)).toBe("const one = t.literal(1)");
        expect(_1.getValidatorsFromString("type f = false", testConfig)).toBe("const f = t.literal(false)");
    });
    test("handles nullable types correctly", function () {
        expect(_1.getValidatorsFromString('type foobar = "foo" | "bar" | null', testConfig)).toBe('const foobar = t.union([t.null, t.literal("foo"), t.literal("bar")])');
    });
});
describe("Configuration", function () {
    test("includeHeader", function () {
        expect(_1.getValidatorsFromString("type a = number;", testConfig)).toBe("const a = t.number");
        expect(_1.getValidatorsFromString("type a = number;", __assign(__assign({}, testConfig), { includeHeader: true }))).toBe('import * as t from "io-ts"\n\nconst a = t.number');
    });
});
describe("Internals", function () {
    test("gets binary flags", function () {
        expect(flags_1.extractFlags(0)).toEqual([]);
        expect(flags_1.extractFlags(1)).toEqual([1]);
        expect(flags_1.extractFlags(10)).toEqual([8, 2]);
        expect(flags_1.extractFlags(100)).toEqual([64, 32, 4]);
        expect(flags_1.extractFlags(67108864)).toEqual([67108864]);
    });
});
//# sourceMappingURL=index.spec.js.map